from os import error
from SEAS.Engine.Core.core import SEAS


class HitboxPoly:
    def __init__(self, points:[]=[], moveHitbox:bool=True)-> None:
        self.inpPoints = points
        self.inpMoveHitbox = moveHitbox

    def start(self) -> None:
        self.points = self.inpPoints
        if self.points == []:
            self.points = SEAS.getScene().getComponent('TransformPoly').points

        self.moveHitbox = self.inpMoveHitbox


    def update(self) -> None:
        if self.moveHitbox:
            self.points = SEAS.getScene().getComponent('TransformPoly').points



from SEAS.Engine.Core import *

import math
import random
import time

class CollidePoly:
    def start(self):
        self.myObj = SEAS.getScene().getComponent('HitboxPoly')
        self.chObj = SEAS.getScene().getObject()

        self.objects = SEAS.getScene().getAllObject()
        self.notTheSame = SEAS.getScene().getAllObject()
        self.realObjects = SEAS.getScene().getAllObject()

        # See if it has the same hitbox
        # WE MUST ADD SO THAT THE OBJECT CAN HAVE MANY MORE HITBOX GROUPS
        for obj in self.notTheSame:
            if not SEAS.sameInitHitboxGroup([obj, self.chObj]):
                self.objects.pop(self.objects.index(obj))
                self.realObjects.pop(self.realObjects.index(obj))

        # Make it all hitboxPoly
        for i in range(len(self.objects)):
            self.objects[i] = self.objects[i].components['HitboxPoly']

        self.collide = [False, None, None, None]

        self.whatSide = None


    def update(self):
        # Update objects
        self.start()
        if SEAS.getHitboxGroupState(SEAS.getObjectInitHitboxGroup(self.chObj)):
            self.updateCorners()

            self.collide = [False, None, None, None]
            for i in range(len(self.sides)): # We want to check myObject and all the other objects
                if self.collide[0] == False:
                    self.mainLoop(self.mySides, self.myCorners, self.sides[i][0], self.corners[i])
                if self.collide[0] == True:
                    self.collide[1] = self.sides[i][1]
                    self.collide[2] = self.sides[i][2]
                    self.collide[3] = self.sides[i][3]
                    break

    def mainLoop(self, mySides, myCorners, objSides, objCorners):
        for mySide in mySides:
            normal = self.normal(mySide)
            scalarsA = [] # First element is p1 and so on
            scalarsB = []

            for myCorner in myCorners:
                scalA = self.dotProduct(normal[1], myCorner) # Project corner onto normal (returns a scalar). Tar slutposen med normalen med [0]
                scalarsA.append(scalA)

            for objCorner in objCorners:
                scalB = self.dotProduct(normal[1], objCorner)
                scalarsB.append(scalB)



            # DEBUG:
            # self.drawNormals(normal, side)


            if self.sortScalar(scalarsA, scalarsB) == False:
                self.collide[0] = False
                self.whatSide = mySide
                break
            else:
                self.collide[0] = True
        
        if self.collide[0] == True:
            for objSide in objSides:
                normal = self.normal(objSide)
                scalarsA = [] # First element is p1 and so on
                scalarsB = []
            

                for myCorner in myCorners:
                    scalA = self.dotProduct(normal[1], myCorner) # Project corner onto normal (returns a scalar). Tar slutposen med normalen med [0]
                    scalarsA.append(scalA)

                for objCorner in objCorners:
                    scalB = self.dotProduct(normal[1], objCorner) # Project corner onto normal (returns a scalar). Tar slutposen av normalen med [0]
                    scalarsB.append(scalB)


                # DEBUG:
                # self.drawNormals(normal, side)

                if self.sortScalar(scalarsA, scalarsB) == False:
                    self.collide[0] = False
                    break
                else:
                    self.collide[0] = True



    def updateCorners(self):
        # THis is the polyCollide so we know that it will be poly
        self.myCorners = self.myObj.points

        self.mySides = []
        for i in range(len(self.myCorners)):
            if i == len(self.myCorners)-1:
                self.mySides.append([self.myCorners[i], self.myCorners[0]])
            else:
                self.mySides.append([self.myCorners[i], self.myCorners[i+1]])

        self.sides = []
        self.corners = []
        # Loop thru objects
        for object, rObject in zip(self.objects, self.realObjects):
            oSide = []
            oCorners = object.points

            for i in range(len(oCorners)):
                if i == len(oCorners)-1:
                    oSide.append([oCorners[i], oCorners[0]])
                else:
                    oSide.append([oCorners[i], oCorners[i+1]])

            self.sides.append([oSide,
                               SEAS.getObjectInitHitboxGroup(rObject),
                               rObject,
                               SEAS.getScene().getRawObjectName(rObject)])
            self.corners.append(oCorners)



    def printCorners(self):
        for corner in self.cornersA:
            pygame.draw.circle(SEAS.coreModules['Screen'].wn, (0, 255, 255), corner, 5, 5)

        for corner in self.cornersB:
            pygame.draw.circle(SEAS.coreModules['Screen'].wn, (0, 255, 255), corner, 5, 5)


    def drawNormals(self, normal, side):
        p1 = side[0] # p1 is not self.p1, its just the first point of the side
        p2 = side[1]

        # Theta: The angle between xplane and normal
        if normal[1][0] != 0:
            theta = math.radians(math.tan(normal[1][1] / normal[1][0]))
        else:
            # Fix this in the future
            theta = 0


        midX = p1[0] - ((p1[0]-p2[0])/2)
        midY = p1[1] - ((p1[1]-p2[1])/2)

        midPos = [midX, midY]

        bNewNormalX = normal[0][0] + midX
        bNewNormalY = normal[0][1] + midY

        eNewNormalX = normal[1][0] + midX
        eNewNormalY = normal[1][1] + midY

        newNormal = [ [bNewNormalX, bNewNormalY], [eNewNormalX, eNewNormalY] ]

        pygame.draw.circle(KEL.coreModules['Screen'].wn, (255, 0, 0), midPos, 5, 5)

        pygame.draw.line(KEL.coreModules['Screen'].wn, (255, 255, 0), newNormal[0], newNormal[1], 1)


    def drawScalars(normal, scalars):
        pass


    def normal(self, v):
        # Because we didnt input a start pos we will just say that its 
        dx = v[1][0] - v[0][0] # dx= x2 - x1 (slutposx - börjposx) 
        dy = v[1][1] - v[0][1] # dy= y2 - y1 (slutposy - börjposy)
        
        return [[-dy, dx], [dy, -dx]] 



    def dotProduct(self, v1, v2): # Inputed only the end point
        return (v1[0] * v2[0]) + (v1[1] * v2[1])
    

    def sortScalar(self, scalarsA, scalarsB):
        minA = min(scalarsA)
        maxA = max(scalarsA)
        
        minB = min(scalarsB)
        maxB = max(scalarsB)

        if maxA > minB and minA < minB:
            return True
        
        if maxB > minA and minB < minA:
            return True


        return False # No collision



from typing import Any
from SEAS.Engine.Setup import *


class Font:
    def start(self):
        self.fonts = {}

    def updateBefore(self): pass
    def update(self): pass

    def createFont(self, fontName:str, fontType:str='freesansbold.ttf', fontSize:int=20) -> None:
        self.fonts[fontName] = pygame.font.Font(fontType, fontSize)
        return self.fonts[fontName]

    def getFont(self, fontName:str) -> Any:
        return self.fonts[fontName]



from SEAS.Engine.Components import *
from SEAS.Engine.Core import *
from SEAS.Engine.Game import *
from SEAS.Engine.Maths import *
from SEAS.Engine.Models import *
from SEAS.Engine.Scene.scene import *



from SEAS.Engine.Core import *


def run():
    SEAS.startCoreObjects()

    run = True
    while run:
        SEAS.updateCore()

        if SEAS.event('type', 'QUIT'):
            run = False



# Start engine when importing
SEAS.startCoreModules()


if __name__ == "__main__":
    run()



from SEAS.Engine.Core import *

class GravityComp:
    def start(self):
        self.velocity = -100
        self.isGround = False


    def update(self):
        transformComp = SEAS.getScene().getComponent('TransformRect')

        # Increasing the increase (accelerating)
        if self.isGround == False:
            self.velocity += 9.8
        else:
            self.velocity = 0

        
        transformComp.yLT += self.velocity * KEL.deltaTime # Deltatime provides us a frame independent game



class EmptyModel:
    def __init__(self):
        self.components = {}

        # This is the currentComponent that is being updated. This is used in the core function getComponent and update given modules
        self.currentComp = None


    def addComponent(self, components, isStart):
        for component in components:
            self.components[type(component).__name__] = component


        if isStart:
            for component in components:
                component.start()

    def removeComponent(self, components):
        for component in components:
            del self.components[type(component).__name__ if type(component) != str else component]

    def start(self):
        for component in self.components:
            self.currentComp = self.components[component]
            self.components[component].start()

    def update(self):
        for component in self.components:
            self.currentComp = self.components[component]
            self.components[component].update()



from SEAS.Engine.Setup import *
from SEAS.Engine.Core.event import *
from SEAS.Engine.Core.screen import *
from SEAS.Engine.Models import *

from typing import Any

import time

class Scene:
    #-----------------------------------------------------------------------FUNC--------------------------------------------------------------
    def __init__(self, frameLimit, wn):
        self.clock = pygame.time.Clock()
        self.frameLimit = frameLimit
        self.framerate = 0
        self.objects = {}
        self.running = False
        self.texts = {}
        self.wn = wn

    #-----------------------------------------------------------------------FUNC--------------------------------------------------------------
    def startObjects(self):
        for component in self.objects:
            self.currentObj = self.objects[component]
            self.objects[component].start()


    #-----------------------------------------------------------------------FUNC--------------------------------------------------------------
    def updateScene(self):
        self.running = True

        # Updating objects, thats really just updating the blueparint that then updates the components of the object
        try:
            for object in self.objects:
                if self.objects != {}:
                    self.currentObj = self.objects[object]
                    self.objects[object].update() 
            
            for text in self.texts:
                if self.texts != {}:
                    t = self.texts[text]
                    if t[1]:
                        self.wn.blit(t[0], t[2])

        except RuntimeError:
            # This just basicly means that if a object was created in the dic we are gonna start OVEERERRERERERERER
            pass

        self.clock.tick(self.frameLimit)


    #-----------------------------------------------------------------------FUNC--------------------------------------------------------------
    def addObject(self, objectName:str="emptyModel", hitbox:bool=True, objectModel:Any=EmptyModel, components:list=[], objectLocation:str='objects') -> None: # Dont use objLocation, hitbox will later be hitbox groups and i t will be in the transform group
        # First get the location by getting the of my self
        location = getattr(self, objectLocation)

        # Then adding it to the attribute
        i = 0
        run = True
        while run:
            i += 1
            if objectName not in self.objects:
                updatedObjectName = objectName
                run = False

            elif objectName + str(i) not in self.objects:
                updatedObjectName = objectName + str(i)
                run = False

        location[updatedObjectName] = objectModel()

        self.currentObj = self.objects[updatedObjectName]

        # Then adding the components we might want to add when we create the object
        self.objects[updatedObjectName].addComponent(components, self.running)


        # Adding a default white material
        self.objects[updatedObjectName].material = "#ffffff"

    def addText(self, font:Any, textName:str, text:str="Your forgor to put a text on the function (addText)", antialias:bool=True, color:str="#000000", backgroundColor:str=None, render:bool=True, position:Any=[0, 0], typePosition:str='center'):
        self.texts[textName] = [font.render(text, antialias, color, backgroundColor), render, None] # None i schanged on the second line
        self.texts[textName][2] = self.texts[textName][0].get_rect() # Make text surface
        if typePosition == 'center':
            self.texts[textName][2].center = position
        else:
            print("other positions not supported yet")

    def updateText(self, font:Any, textName:str, text:str, antialias:bool=True, color:str="#ffffff", backgroundColor=None):
        self.texts[textName][0] = font.render(
                                text,
                                antialias,
                                color,
                                backgroundColor)
        return self.texts[textName]

    def removeObject(self) -> None:
        for key, value in self.objects.items():
            if value == self.currentObj:
                del self.objects[key]

    def removeAllObject(self) -> None:
        self.objects = {}

    def removeRawNameObject(self, objectName) -> None:
        del self.objects[objectName]

    def removeRawInitObject(self, objectInit) -> None:
        for value, key in self.objects.items():
            if key == objectInit:
                del self.objects[value]

    def removeAllText(self) -> None:
        self.texts = {}

    def removeText(self, textName) -> None:
        del self.texts[textName]

    #-----------------------------------------------------------------------FUNC--------------------------------------------------------------
    def getComponent(self, attribute=''):
        # If nothing is specified return the object u r using
        if attribute == '':
            return self.currentObj
        

        # If it doesnt have the attribute just return the AttributeError 
        try:
            returnValue = self.currentObj.components[attribute]

            return returnValue  

        except AttributeError as err:
            raise err

    #-----------------------------------------------------------------------FUNC--------------------------------------------------------------
    def getRawComponent(self, object:str, attribute:str=''):
        # So were basicly doing getAttribute function but we specify the object and do not use the currentObj
        if attribute == '':
            return self.objects[object]
        try:
            returnValue = self.objects[object].components[attribute]

            return returnValue

        except AttributeError as err:
            raise err


    #-----------------------------------------------------------------------FUNC--------------------------------------------------------------
    def getObject(self):
        return self.currentObj

    def getRawObject(self, object:str):
        return self.objects[object]
    
    def getAllObject(self): # Will not return the obejct your calling from
        returnValue = []
        for obj in self.objects:
            if self.objects[obj] != self.currentObj:
                returnValue.append(self.objects[obj])

        return returnValue
    
    #-----------------------------------------------------------------------FUNC--------------------------------------------------------------
    def getAttribute(self, attribute):
        # Get the requested attribute of the current object

        try:
            returnValue = getattr(self.currentObj, attribute)
            return returnValue
        
        except AttributeError as err:
            raise err


    #-----------------------------------------------------------------------FUNC--------------------------------------------------------------
    def getRawAttribute(self, object, attribute):
        # Get the requested attribute of the object requested

        try:
            returnValue = getattr(self.objects[object], attribute)
            return returnValue

        except AttributeError as err:
            raise err

    def getObjectName(self):
        return list(self.objects.keys())[list(self.objects.values()).index(self.currentObj)]

    def getRawObjectName(self, name):
        return list(self.objects.keys())[list(self.objects.values()).index(name)]



class TransformRect:
    def __init__(self, xLT=100, yLT=100, width=100, height=100) -> None:
        self.xLT = xLT
        self.yLT = yLT 
        self.width = width 
        self.height = height

    def start(self):
        pass

    def update(self):
        pass



from SEAS.Engine.Setup import *
from SEAS.Engine.Core import *


class RenderRect:
    def start(self):
        self.transformComp = SEAS.getScene().getComponent('TransformRect')

    def update(self):
        if self.transformComp.isVisible:
            self.objectColor = SEAS.getMaterial()
            lX, lY, w, h = self.transformComp.xLT, self.transformComp.yLT, self.transformComp.width, self.transformComp.height
            pygame.draw.rect(SEAS.coreModules['Screen'].wn, self.objectColor, pygame.Rect((lX, lY), (w, h)))



from SEAS.Engine.Core.core import SEAS


class TransformPoly:
    def __init__(self, point, angle=0):
        # Make so points are relavant to other points

        # We do this because the currentObj is not updated yet. This wont do a diff here but its good practise (look at hitboxPoly)
        self.inpPoint = point
        self.inpAngle = angle

    def start(self):
        self.points = self.inpPoint
        self.angle = self.inpAngle
        
        self.isVisible = False
        for p in self.points:
            if (0 < p[0] < SEAS.getCoreModule('Screen').wW) and (0 < p[1] < SEAS.getCoreModule('Screen').wH):
                self.isVisible = True
                break

    def update(self):
        self.isVisible = False
        for p in self.points:
            if (0 < p[0] < SEAS.getCoreModule('Screen').wW) and (0 < p[1] < SEAS.getCoreModule('Screen').wH):
                self.isVisible = True
                break



from SEAS.Engine.Core import *
from SEAS.Engine.Setup import *

import math


class CharacterPolyController:
    def start(self): 
        self.trns = SEAS.getScene().getComponent('TransformPoly')
        self.hitb = SEAS.getScene().getComponent('HitboxPoly')
        
    def update(self): 
        pass

    def move(self, vel, angle='default'):
        if angle == 'default':
            nAngle = self.trns.angle
        else:
            nAngle = angle

        mX = math.cos(math.radians(nAngle)) * vel
        mY = math.sin(math.radians(nAngle)) * vel

        self.moveX(mX)
        self.moveY(mY)

    # make so that points move in the future
    def rawRotate(self, points, angle, angleChange, _axis='centroid'):
        if angleChange == 0: return [points, angle]
        angle += angleChange

        if _axis == 'centroid': axis = self.__axis(points)
        else: axis = _axis

        newPoints = []
        for p in points: newPoints.append(self.rotatePoint(axis[0], axis[1], angleChange, p))
        for p, i in zip(newPoints, range(len(points))): points[i] = p

        return [points, angle]
        
    def rotate(self, angle, _axis='centroid'):
        if angle == 0: return None
        self.trns.angle += angle

        if _axis == 'centroid': axis = self.__axis(self.trns.points)
        else: axis = _axis

        newPoints = []
        for p in self.trns.points: newPoints.append(self.rotatePoint(axis[0], axis[1], angle, p))
        for p, i in zip(newPoints, range(len(self.trns.points))): self.trns.points[i] = p

    def __sin(self, angle): return (math.sin(math.radians(angle)))
    def __cos(self, angle): return (math.cos(math.radians(angle)))

    def rotatePoint(self, cx, cy, angle, p):
        return [self.__cos(angle) * (p[0] - cx) - self.__sin(angle) * (p[1] - cy) + cx,
                  self.__sin(angle) * (p[0] - cx) + self.__cos(angle) * (p[1] - cy) + cy]

    def __axis(self, points):
        xPoints = []
        for p in points:
            xPoints.append(p[0])
        yPoints = []
        for p in points:
            yPoints.append(p[1])
        cX = sum(xPoints)/len(xPoints)
        cY = sum(yPoints)/len(yPoints)
        axis = [cX, cY]
        return axis
    
    def drawDirection(self):
        len = 50
        lenK1 = math.cos(math.radians(self.trns.angle))*len
        lenK2 = math.sin(math.radians(self.trns.angle))*len

        
        pygame.draw.line(SEAS.coreModules['Screen'].wn,
                (0, 255, 0), 
                (self.trns.points[0][0], self.trns.points[0][1]), 
                (lenK1+self.trns.points[0][0], lenK2+self.trns.points[0][1]) )

    def moveX(self, vel):
        for point in self.trns.points:
            point[0] += vel

    def moveY(self, vel):
        for point in self.trns.points:
            point[1] += vel



from SEAS.Engine.Setup import *


class Event:
    def start(self):
        pass

    def updateBefore(self): # Check screen for descrition
        self.events = []

        for event in pygame.event.get():
            self.events.append(event)



from SEAS.Engine.Scene.scene import *

from SEAS.Engine.Core.event import *
from SEAS.Engine.Core.screen import *
from SEAS.Engine.Core.filePreset import *
from SEAS.Engine.Core.input import *
from SEAS.Engine.Core.font import *


from typing import Any


class GameCore:
    def __init__(self) -> None:
        self.scenes = {}
        self.targetedScene = None

        self.materials = {}
        self.coreModules = {'Event': Event(), 'Input': Input(),'Screen': Screen(), 'Font': Font()}
        self.hitboxGroup = {}

        self.inputStateDefault = 'Up'

        self.printed1 = 0
        self.printed2 = 0

        self.deltaTime = 0
        self.time1 = 0
        self.time2 = 0

        self.loggingLevel = 'Comment'


    def startCoreModules(self) -> None:
        for mod in self.coreModules:
            self.coreModules[mod].start()

    def startCoreObjects(self) -> None: #The scenes work. Only will run when we hit run() or when a new scene is created
        try:
            self.targetedScene.startObjects()
        except AttributeError as err:
            print("DISCLAMER: This could be because you have not created a scene yet. Pls proceed and check that thats intact!")
            print("\n\n\n\n")
            raise err

    def updateCore(self) -> None:
        self.time1 = time.time()

        # bef
        for module in self.coreModules:
            self.coreModules[module].updateBefore()

        try:
            self.targetedScene.updateScene()
        except AttributeError as err:
            print("DISCLAMER: This could be because you have not created a scene yet. Pls proceed and check that thats intact!")
            print("\n####################\n####################\n####################\n####################\n")
            raise err
        
        pygame.display.flip()

        self.time2 = time.time()
        self.deltaTime = self.time2 - self.time1

    def getCoreModule(self, coreModName:str=''):
        if coreModName == '':
            print('SEAS CORE::getCoreModule(): ERROR: U need to specify a coreModName')
        
        try:
            return self.coreModules[coreModName]
        except:
            # Doesnt need to check any loggingLevel
            print('SEAS CORE::getCoreModule(): ERROR: Not a valid coreModuleName')


    def createFilePreset(self, fileName:str) -> None: # This is only experimental
        with open('SEAS/Engine/Core/filePreset.py', 'r') as f:
            filePreset = f.read()
            # Returns the index that 'n' is on
            indexN = filePreset.find('name')
            
            filePreset2 = filePreset[:indexN] + fileName + filePreset[indexN+4:]

        with open(fileName + '.py', 'w+') as f:
            f.write(filePreset2)

    def addScene(self, name:str, frameLimit:int=60, isTargeted:bool=True, overflowObj:str='') -> Any:
        self.scenes[name] = Scene(frameLimit, self.getCoreModule('Screen').wn)

        if isTargeted:
            self.targetedScene = self.scenes[name]

        return self.scenes[name]


    def targetScene(self, sceneName:str) -> None:
        if self.scenes[sceneName] != self.targetScene:
            sceneBef = self.targetedScene
            self.targetedScene = self.scenes[sceneName]
            self.startCoreModules()
            self.targetedScene.currentObj = sceneBef.currentObj

    def getScene(self) -> Any:
        return self.targetedScene

    def getRawScene(self, sceneName:str) -> Any:
        return self.scenes[sceneName]

    def getAllScene(self) -> Any:
        return self.scenes

    def transferObject(self, objectName:str, sceneName:str, resetObject:bool=False) -> None:
        if not (objectName in self.scenes[sceneName].objects):
            self.scenes[sceneName].objects[objectName] = self.targetedScene.objects[objectName]
            
            if resetObject:
                self.scenes[sceneName].objects[objectName].start()
                # TODO: Move object to start positions

    def transferRawObject(self, objectName:str, fromScene:str, toScene:str, resetObject:bool=False) -> None:
        self.scenes[toScene].objects[objectName] = self.scenes[fromScene].objects[objectName]

        if resetObject:
            self.scenes[toScene].objects[objectName].start()
            # TODO: Move object to start positions
    
    def transferAllObject(self, sceneName:str):
        print("transferAllObject() function not implemented yet coming soon sry")

    def transferRawAllObject(self, fromScene:str, toScene:str):
        print("transferRawAllObject() function not implemented yet coming soon sry")


    def createMaterial(self, materialName:str, materialColor:str) -> None: # Color will be hexadecimal
        self.materials[materialName] = materialColor

    def addMaterial(self, materialName:str, objectName:str) -> None:
        material = self.materials[materialName]
        object = self.getScene().objects[objectName]

        object.material = material

    def getMaterial(self) -> str: # This is the material that the current obj has
        return self.getScene().currentObj.material

    def getRawMaterial(self, materialName):
        return self.materials[materialName]

    def getObjectMaterial(self, objectName):
        return self.targetScene.objects[object].material

    def input(self, key:str) -> bool:
        indexAttr = getattr(pygame, 'K_' + key)
        if self.coreModules['Input'].keys[indexAttr]:
            return True
        return False

    def event(self, eventType:str, eventEquals:str) -> bool:
        for event in self.coreModules['Event'].events: # Loop thru events list
            attr = getattr(event, eventType)
            equals = getattr(pygame, eventEquals)

            if attr == equals:
                return True

        return False

    def createHitboxGroup(self, groupName, state=False):
        self.hitboxGroup[groupName] = [[], state]

    def addRawNameHitboxGroup(self, groupName:str, objects:list=[]): # Call this only in a component
        for obj in objects:
            # oAdd.append(self.getScene().objects[obj])
            self.hitboxGroup[groupName][0].append(self.getScene().objects[obj])

        # self.hitboxGroup[groupName][0] = oAdd

    def addRawInitHitboxGroup(self, groupName:str, objects:list=[]): # Call this only in a component
        for obj in objects:
            self.hitboxGroup[groupName][0].append(obj)

    def toggleHitboxGroup(self, groupName:str):
        if self.hitboxGroup[groupName][1] == False:
            self.hitboxGroup[groupName][1] = True
        else:
            self.hitboxGroup[groupName][1] = False

    def getHitboxGroupState(self, groupName:str):
        try:
            return self.hitboxGroup[groupName][1]
        except:
            return False

    def getObjectNameHitboxGroup(self, objectName:str):
        for g in self.hitboxGroup:
            if objectName in self.hitboxGroup[g][0]:
                return g

    def getObjectInitHitboxGroup(self, objectInit:str):
        for g in self.hitboxGroup:
            if objectInit in self.hitboxGroup[g][0]:
                return g

    def sameNameHitboxGroup(self, objectNames:list) -> bool:
        # We need to match one with the other 
        objGroup = []
        for obj in objectNames:
            objH = []
            for h in self.hitboxGroup:
                if self.getScene().objects[obj] in self.hitboxGroup[h][0]:
                    objH.append(h)
            objGroup.append(objH)
        
        for obj1 in objGroup:
            for obj2 in objGroup:
                for i in obj1:
                    for j in obj2:
                        if obj1 != obj2:
                            if i == j:
                                return True
    def sameInitHitboxGroup(self, objectInits:list) -> bool:
        # We need to match one with the other 
        objGroup = []
        for obj in objectInits:
            objH = []
            for h in self.hitboxGroup:
                if obj in self.hitboxGroup[h][0]:
                    objH.append(h)
            objGroup.append(objH)
        
        for obj1 in objGroup:
            for obj2 in objGroup:
                for i in obj1:
                    for j in obj2:
                        if obj1 != obj2:
                            if i == j:
                                return True

    def same(self, iterator):
        iterator = iter(iterator)
        try:
            first = next(iterator)
        except StopIteration:
            return True
        return all(first == x for x in iterator)

    
SEAS = GameCore()



from setuptools import setup, find_packages
import codecs
import os


########SAVE THIS UNVALUABLE INFORMATION#########
#      python3 setup.py sdist bdist_wheel       #
#        twine upload dist/* --verbose          #
########SAVE THIS UNVALUABLE INFORMATION#########


VERSION = '1.0.5'
DESCRIPTION = 'A Simple Game Engine Libary'
LONG_DESCRIPTION = 'Chech github: https://github.com/coding610/SEAS/tree/master'


# Setting up
setup(
    name="MySEAS_OLD",
    version=VERSION,
    author="Sixten Bohman",
    author_email="sixten.bohman.08@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,

    packages=find_packages(),
    
    install_requires=['pygame'],
    keywords=['python', 'pygame', 'Game Engine', 'Simple', 'Beginner', 'Learning', 'Funny?'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



from SEAS.Engine.Setup import *


class Screen:
    def start(self):
        # 160 	× 	120
        self.wW = 1000 
        self.wH = 1000 
        self.wn = pygame.display.set_mode((self.wW, self.wH))


        # Frame Related
        self.clock = pygame.time.Clock()
        self.frameLimit = 60
        self.frameRate = self.clock.get_fps()


        self.color = "#ffffff"


    def updateBefore(self): # Before updating objects
        self.wn.fill(self.color)

        # Clock
        self.clock.tick(self.frameLimit)
        self.frameRate = self.clock.get_fps()



from SEAS.Engine.Components.renderRectComp import *
from SEAS.Engine.Components.transformRectComp import *
from SEAS.Engine.Components.gravityComp import *
from SEAS.Engine.Components.renderPolyComp import *
from SEAS.Engine.Components.transformPolyComp import *
from SEAS.Engine.Components.collidePolyComp import *
from SEAS.Engine.Components.characterPolyControllerComp import *
from SEAS.Engine.Components.hitboxPolyComp import *



from SEAS.Engine.Setup import *
from SEAS.Engine.Core import *


class RenderPoly:
    def start(self):
        self.transformComp = SEAS.getScene().getComponent('TransformPoly')

    def update(self):
        if self.transformComp.isVisible:
            objectColor = SEAS.getMaterial()
            points = self.transformComp.points
            pygame.draw.polygon(SEAS.coreModules['Screen'].wn, objectColor, points)
